//
//  HelloWorldLayer.m
//  CatJump
//
//  Created by Ray Wenderlich on 3/22/11.
//  Copyright Ray Wenderlich 2011. All rights reserved.
//


#import "HelloWorldLayer.h"
#import "SimpleAudioEngine.h"
#import "GameOverScene.h"

@interface HelloWorldLayer() {
    CCSprite * _cat;
    CCNode *_vehicles;
    BOOL _invincible;
    BOOL _jumping;
    double _nextSpawn;
    int _lives;
    int _dodges;
    CCSpriteBatchNode *_catJumpBatchNode;
    CCAnimation *_catJumpAnimation;
}
@property (retain) HelloWorldHud *hud;
@end

@implementation HelloWorldLayer

// on "init" you need to initialize your instance
-(id) initWithHud:(HelloWorldHud *)myHud 
{
	// always call "super" init
	// Apple recommends to re-assign "self" with the "super" return value
	if( (self=[super init])) {
        
        self.hud = myHud;
		
        CCSprite *bg = [CCSprite spriteWithFile:@"bg.png"]; 
        //bg.position = ccp(winSize.width/2, winSize.height/2);
        bg.anchorPoint = ccp(0,0);
        [self addChild:bg];
        
        [[CCSpriteFrameCache sharedSpriteFrameCache] addSpriteFramesWithFile:@"CatJumpAtlas.plist"];
        
        _catJumpBatchNode = [CCSpriteBatchNode batchNodeWithFile:@"CatJumpAtlas.png"];
        [self addChild:_catJumpBatchNode];
        
        _cat = [CCSprite spriteWithSpriteFrameName:@"cat_stand_1.png"];
        _cat.position = ccp(75,75);
        [_catJumpBatchNode addChild:_cat z:1 tag:1];

        
        NSArray *spriteAnimFrames = [NSArray arrayWithObjects:
                                     [[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:@"cat_leap_1.png"],
                                     [[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:@"cat_leap_2.png"],
                                     nil];
        _catJumpAnimation = [CCAnimation animationWithSpriteFrames:spriteAnimFrames delay:0.625f];
        [_catJumpAnimation retain];

        // If you want to add this to the AnimationCache instead of retaining
        //[[CCAnimationCache sharedAnimationCache] addAnimation:catJumpAnimation name:@"catJumpAnim"];
        
        // Dog Animation
        NSArray *dogAnimFrames = [NSArray arrayWithObjects:
                                  [[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:@"dog_1.png"],
                                  [[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:@"dog_2.png"],
                                  [[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:@"dog_3.png"],
                                  [[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:@"dog_4.png"], nil];
        CCAnimation *dogAnimation = [CCAnimation animationWithSpriteFrames:dogAnimFrames delay:1.0f/dogAnimFrames.count];
        [[CCAnimationCache sharedAnimationCache] addAnimation:dogAnimation name:@"dogAnimation"];
        
        // Kid Animation
        NSArray *kidAnimFrames = [NSArray arrayWithObjects:
                                  [[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:@"kidontrike_1.png"],
                                  [[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:@"kidontrike_2.png"],
                                  [[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:@"kidontrike_3.png"],
                                  [[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:@"kidontrike_4.png"], nil];
        CCAnimation *kidAnimation = [CCAnimation animationWithSpriteFrames:kidAnimFrames delay:1.0f/kidAnimFrames.count];
        [[CCAnimationCache sharedAnimationCache] addAnimation:kidAnimation name:@"kidAnimation"];
        
        
       // Older code before using CCSpriteBatchNode 
//        cat = [CCSprite spriteWithFile:@"cat_stand_1.png"];
//        cat.position = ccp(75, 75);
//        [self addChild:cat];
        
        /*
        CCAction *jumpAction = [CCJumpBy actionWithDuration:1.0 position:ccp(0,0) height:200 jumps:1];
        CCAction *repeatAction = [CCRepeatForever actionWithAction:jumpAction];
		[cat runAction:repeatAction];
        */
        
        self.isTouchEnabled = YES;
        
        //[[SimpleAudioEngine sharedEngine] preloadEffect:@"meow.wav"];
        
        [self scheduleUpdate];
        
        _vehicles = [CCNode node];
        [self addChild:_vehicles];
        
        _lives = 9;
        _dodges = 0;
        [_hud setLives:_lives];
        [_hud setDodges:_dodges];
        
        
        double curTime = [[NSDate date] timeIntervalSince1970];
        _nextSpawn = curTime + 4;
	}
	return self;
}

- (void)carDone:(id)sender {
    
    CCSprite *vehicle = (CCSprite *)sender;
    [vehicle removeFromParentAndCleanup:YES];
    
    _dodges++;
    [_hud setDodges:_dodges];
}

- (void)doneInvincible {
    _invincible = FALSE;
}

- (void)update:(ccTime)dt {
   CGSize winSize = [CCDirector sharedDirector].winSize;
    CCSprite *vehicleSprite;
    // Spawn Vehicles (new)
    double curTime = [[NSDate date] timeIntervalSince1970];
    if (curTime > _nextSpawn) {
        
        int randomVehicle = arc4random() % 4;
        
        if (randomVehicle == kVehicleTypeRedCar) {
            // Red Car
            vehicleSprite = [CCSprite spriteWithSpriteFrame:[[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:@"car1_1.png"]];
            [vehicleSprite setUserData:[NSNumber numberWithInt:kVehicleTypeRedCar]];
            CCSprite *wheel1 = [CCSprite spriteWithSpriteFrame:[[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:@"car1_tire.png"]];
            CCSprite *wheel2 = [CCSprite spriteWithSpriteFrame:[[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:@"car1_tire.png"]];
            id tireRotateAction1 = [CCRotateBy actionWithDuration:1.0f angle:360.0f]; // Ray, these are backwards on purpose as a lab exercise.
            id tireRotateAction2 = [CCRotateBy actionWithDuration:1.0f angle:360.0f];
            [wheel1 runAction:[CCRepeatForever actionWithAction:tireRotateAction1]];
            [wheel2 runAction:[CCRepeatForever actionWithAction:tireRotateAction2]];
            [vehicleSprite addChild:wheel1];
            [vehicleSprite addChild:wheel2];
            [wheel1 setPosition:ccp(65,18)];
            [wheel2 setPosition:ccp(212,18)];
            
        } else if (randomVehicle == kVehicleTypeYellowCar) {
            // Yellow Car (Same code as Red Car except for wheel placement, re-listed for clarity. Consilidate in your own games)
            vehicleSprite = [CCSprite spriteWithSpriteFrame:[[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:@"car2_1.png"]];
            [vehicleSprite setUserData:[NSNumber numberWithInt:kVehicleTypeYellowCar]];
            CCSprite *wheel1 = [CCSprite spriteWithSpriteFrame:[[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:@"car2_tire.png"]];
            CCSprite *wheel2 = [CCSprite spriteWithSpriteFrame:[[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:@"car2_tire.png"]];
            id tireRotateAction1 = [CCRotateBy actionWithDuration:1.0f angle:-360.0f];
            id tireRotateAction2 = [CCRotateBy actionWithDuration:1.0f angle:-360.0f];
            [wheel1 runAction:[CCRepeatForever actionWithAction:tireRotateAction1]];
            [wheel2 runAction:[CCRepeatForever actionWithAction:tireRotateAction2]];
            [vehicleSprite addChild:wheel1];
            [vehicleSprite addChild:wheel2];
            [wheel1 setPosition:ccp(62,15)];
            [wheel2 setPosition:ccp(195,15)];
    
        } else if (randomVehicle == kVehicleTypeDog) {
            // Dog
            vehicleSprite = [CCSprite spriteWithSpriteFrame:[[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:@"dog_1.png"]];
            [vehicleSprite setUserData:[NSNumber numberWithInt:kVehicleTypeDog]];
             
            // In your code, check that the animationByName did not return nil (due to memory warnings)
            CCAnimation *vehicleAnimation = [[CCAnimationCache sharedAnimationCache] animationByName:@"dogAnimation"];
            
            CCAnimate *vehicleAnimate = [CCAnimate actionWithAnimation:vehicleAnimation];
            [vehicleSprite runAction:[CCRepeatForever actionWithAction:vehicleAnimate]];
            
        } else {
            // Kid on Bike (Same code as Dog, re-listed for clarity. Consilidate in your own games)
            vehicleSprite = [CCSprite spriteWithSpriteFrame:[[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:@"kidontrike_1.png"]];
            [vehicleSprite setUserData:[NSNumber numberWithInt:kVehicleTypeKid]];
            
            // In your code, check that the animationByName did not return nil (due to memory warnings)
            CCAnimation *vehicleAnimation = [[CCAnimationCache sharedAnimationCache] animationByName:@"kidAnimation"];
            
            CCAnimate *vehicleAnimate = [CCAnimate actionWithAnimation:vehicleAnimation];
            [vehicleSprite runAction:[CCRepeatForever actionWithAction:vehicleAnimate]];
        }
        
        // Common placement and movement code for all vehicles
        
        vehicleSprite.position = ccp(winSize.width + vehicleSprite.contentSize.width/2, 75);
        [_catJumpBatchNode addChild:vehicleSprite];
        
        [vehicleSprite runAction:[CCSequence actions:
                                  [CCMoveBy actionWithDuration:1.25 position:ccp(-winSize.width-vehicleSprite.contentSize.width, 0)],
                                  [CCCallFuncN actionWithTarget:self selector:@selector(carDone:)],
                                  nil]];
        
        float randomInterval = arc4random() % 3 + 1.5;
        _nextSpawn = curTime + randomInterval;
    }

    
        // Check for collisions
        if (!_invincible) {
            float insetAmtX = 10;
            float insetAmtY = 10;
            BOOL isCatColliding;
            CGRect catRect = CGRectInset(_cat.boundingBox, insetAmtX, insetAmtY);
            CGRect vehicleRect;
            for (CCSprite *vehicle in _catJumpBatchNode.children) {
                if ([vehicle tag] == 1) {
                    continue;  // No need to check if the Cat collides with itself
                }
                isCatColliding = NO;
                NSNumber *vehicleTypeNumber = (NSNumber*)[vehicle userData];
                int vehicleType = [vehicleTypeNumber intValue];
                
                if (vehicleType == kVehicleTypeRedCar) {
                    CGPoint boundingBoxOrigin = vehicle.boundingBox.origin;
                    CGRect carHood = CGRectMake(boundingBoxOrigin.x+10,boundingBoxOrigin.y , 40,80);
                    insetAmtX = 50;
                    insetAmtY = 10;
                    vehicleRect = CGRectInset(vehicle.boundingBox,insetAmtX,insetAmtY);
                        
                   if ((CGRectIntersectsRect(catRect,carHood)) ||
                       (CGRectIntersectsRect(catRect, vehicleRect))) {
                       isCatColliding = YES;
                       CCLOG(@"Collided with Red Car");
                   }
                    
                } else if (vehicleType == kVehicleTypeYellowCar) {
                    CGPoint boundingBoxOrigin = vehicle.boundingBox.origin;
                    CGRect carHood = CGRectMake(boundingBoxOrigin.x+10,boundingBoxOrigin.y , 68,65);
                    insetAmtX = 68;
                    insetAmtY = 10;
                    vehicleRect = CGRectInset(vehicle.boundingBox,insetAmtX,insetAmtY);
                    
                    if ((CGRectIntersectsRect(catRect,carHood)) ||
                        (CGRectIntersectsRect(catRect, vehicleRect))) {
                        isCatColliding = YES;
                        CCLOG(@"Collided with Yellow Car");
                    } 
                    
                    
                } else {
                    // Dog or Kid
                    CGRect vehicleRect = CGRectInset(vehicle.boundingBox, insetAmtX, insetAmtY);
                    if (CGRectIntersectsRect(catRect, vehicleRect)) {
                        isCatColliding = YES;
                    }
                }
                
                
                if (isCatColliding == YES) {
                    // Play sound, take a hit, invincible, break out of the loop
                    [[SimpleAudioEngine sharedEngine] playEffect:@"squish.wav"];            
                    _invincible = TRUE;
                    [_cat runAction:[CCSequence actions:
                           [CCBlink actionWithDuration:1.0 blinks:6],
                           [CCCallFunc actionWithTarget:self selector:@selector(doneInvincible)],
                           nil]];     
                    _lives--;
                    [_hud setLives:_lives];
                                        
                    if (_lives <= 0) {
                        GameOverScene *scene = [[[GameOverScene alloc] initWithDodges:_dodges] autorelease];
                        [[CCDirector sharedDirector] replaceScene:scene];
                    }
                    break;
                }
            }
            
        }
}

- (void)doneJump {
    _jumping = FALSE;
}

- (void)ccTouchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
    
    if (!_jumping) {
        
        _jumping = TRUE;
        [[SimpleAudioEngine sharedEngine] playEffect:@"meow.wav"];
        
        // If you did not retain the CCAnimation in the initWithHUD and 
        // wanted to retrieve from the Animation Cache
        //CCAnimate *jumpAnimation = [CCAnimate actionWithAnimation:catJumpAnimation restoreOriginalFrame:YES];
        
        CCLOG(@"Making the Cat Jump");
        _catJumpAnimation.restoreOriginalFrame = YES;
        CCAnimate *jumpAnimation = [CCAnimate actionWithAnimation:_catJumpAnimation];
        CCJumpBy *jumpAction = [CCJumpBy actionWithDuration:1.25 position:ccp(0,0) height:200 jumps:1];
        CCCallFunc *doneJumpAction = [CCCallFunc actionWithTarget:self selector:@selector(doneJump)];
        CCSequence *sequenceAction = [CCSequence actions:jumpAction,doneJumpAction, nil];

		[_cat runAction:[CCSpawn actions:jumpAnimation,sequenceAction, nil]];
    }
    
}

// on "dealloc" you need to release all your retained objects
- (void) dealloc
{
	// in case you have something to dealloc, do it in this method
	// in this particular example nothing needs to be released.
	// cocos2d will automatically release all the children (Label)
	[_hud release];
    _hud = nil;
    
	// don't forget to call "super dealloc"
	[super dealloc];
}
@end
